﻿using System;

namespace UMK.WzorceStrukturalne
{
    //component
    abstract class Napój
    {
        public abstract decimal Cena();
        public void Objętość() //funckja nie podlegająca zmianom podczas dekoracji
        {
            Console.WriteLine("\nObjetosc: 330 ml");
        }
    }

    //concrete components
    class Herbata : Napój
    {
        public override decimal Cena()
        {
            return 6.70M;
        }

        public override string ToString()
        {
            return "Herbata";
        }
    }

    class Kawa : Napój
    {
        public override decimal Cena()
        {
            return 7.90M;
        }

        public override string ToString()
        {
            return "Kawa";
        }
    }

    //decorator
    abstract class NapójZDodatek : Napój
    {
        protected Napój napój;

        public NapójZDodatek(Napój napój)
        {
            this.napój = napój;
        }
    }

    //concrete decorator
    class NapójZeSzczyptąCynamonu : NapójZDodatek
    {
        public NapójZeSzczyptąCynamonu(Napój napój)
            : base(napój)
        { }

        public override decimal Cena()
        {
            return napój.Cena() + 0.5M;
        }

        public override string ToString()
        {
            return napój.ToString() + " ze szczyptą cynamonu";
        }
    }

    class NapójZCukrem : NapójZDodatek
    {
        private int liczbaŁyżeczek;

        public NapójZCukrem(Napój napój, int liczbaŁyżeczek = 1)
            :base(napój)
        {
            this.liczbaŁyżeczek = liczbaŁyżeczek;
        }

        public override decimal Cena()
        {
            return napój.Cena() + liczbaŁyżeczek * 0.2M;
        }

        public override string ToString()
        {
            return napój.ToString() + $" z {liczbaŁyżeczek} łyżeczkami cukru";
        }
    }
}
